--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--CONTROL4 - LUTRON RA2 SELECT FAN
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

MAIN_BINDING        = 1
FAN_PROXY_BINDING = 5001
OFF_BINDING          = 200
LOW_BINDING          = 201
MED_BINDING          = 202
MED_HI_BINDING      = 203
HI_BINDING          = 204
TOGGLE_BINDING      = 205
CYCLE_BINDING       = 206

gFanSpeed = 0


function OnDriverLateInit()
    if (PersistData == nil or PersistData["PresetSpeed"] == nil) then
        PersistData = {}
        PersistData["PresetSpeed"] = 4
    end

   SendToProxy(FAN_PROXY_BINDING, "PRESET_SPEED", {SPEED = PersistData["PresetSpeed"]}) 

   g_GetSpeedTimer = C4:AddTimer(30, "SECONDS")
end


--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--PROPERTIES HANDLER
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

function OnPropertyChanged(strProperty)
    dbg("strProperty = "  .. strProperty .. " changed to: " .. Properties[strProperty])
    local strProp = string.gsub(strProperty, " ", "")
    if (PROPS[strProp] ~= nil and type(PROPS[strProp]) == "function") then
        PROPS[strProp](Properties[strProperty])
    else
        dbg("UNKNOWN | THROW AWAY: " .. strProperty)
    end
end

PROPS = {}

function PROPS.DebugMode(value)
    if (value == "Off") then
        g_debugprint = false
        g_debugunittest = false
        g_debuglog = false
        g_DebugTimer = 0
    elseif (value == "Print") then
        g_debugprint = true
        g_debugunittest = false
        g_debuglog = false
        startDebugTimer()
    elseif (value == "Log") then
        g_debugprint = false
        g_debugunittest = true
        g_debuglog = true
        startDebugTimer()
    elseif (value == "Print and Log") then
        g_debugprint = true
        g_debugunittest = true
        g_debuglog = true
        startDebugTimer()
    end
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--TIMER CALLBACKS
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

function OnTimerExpired(idTimer)
    if (idTimer == g_DebugTimer) then
        dbg("Turning Debug Mode Off (timer expired)")
        C4:UpdateProperty("Debug Mode", "Off")
    elseif (idTimer == g_GetSpeedTimer) then
        GET_FAN_SPEED()
        C4:KillTimer(g_GetSpeedTimer)
    else
        dbg('OnTimerExpired', idTimer)
        local timer = pullTimer(idTimer)
        if (not timer) then dbg("UNKNOWN TIMER FIRED | THROWING AWAY") return end
        timer.handler()
    end
end

----

g_timers = {}

function startTimer(value, units, callback)
    local timerId = C4:AddTimer(tonumber(value), units)
    g_timers[timerId] = { handler = callback }
    return timerId
end

function pullTimer(timerId)
    local timer = g_timers[timerId]
    g_timers[timerId] = nil
    return timer
end

function killTimer(timerId)
    local timer = g_timers[timerId]
    if (timer) then
        C4:KillTimer(timerId)
        g_timers[timerId] = nil
    end
    return timer
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--PROXY MESSAGE HANDLER
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

function SendToProxy(idBinding, strCommand, tParams, ...)
    dbgFunction("SendToProxy (" .. idBinding .. ", " .. strCommand .. ")")
    C4:SendToProxy(idBinding, strCommand, tParams)
end

function ReceivedFromProxy(idBinding, strCommand, tParams)
    dbgFunction("ReceivedFromProxy (" .. idBinding .. ", " .. strCommand .. ")")

    if (tParams == nil) then
        tParams = {}
    end

    if (idBinding == FAN_PROXY_BINDING) then
         if (strCommand == "TOGGLE") then
            TOGGLE_PRESET()

        elseif (strCommand == "SET_SPEED") then
          for k,v in pairs(tParams) do
            dbg(k .. ": " .. v)
          end 
          SET_SPEED({SPEED=tonumber(tParams["SPEED"]) * 25})

        elseif(strCommand == "DESIGNATE_PRESET") then
            PersistData["PresetSpeed"] = tonumber(tParams["PRESET"])
            SendToProxy(FAN_PROXY_BINDING, "PRESET_SPEED", {SPEED = PersistData["PresetSpeed"]})

        elseif (strCommand == "ON") then
            SET_SPEED({SPEED = PersistData["PresetSpeed"] * 25})
            SendToProxy(FAN_PROXY_BINDING, "ON", {})

        elseif (strCommand == "OFF") then
            SET_SPEED({SPEED = 0})
            SendToProxy(FAN_PROXY_BINDING, "OFF", {})

        elseif (strCommand == "CYCLE_SPEED_UP") then
            if (tonumber(gFanSpeed) + 25 <= 100) then
                SET_SPEED({SPEED=tonumber(gFanSpeed) + 25})
            end

        elseif (strCommand == "CYCLE_SPEED_DOWN") then
            if (tonumber(gFanSpeed) - 25 >= 0) then
                SET_SPEED({SPEED=tonumber(gFanSpeed) - 25})
            end
        end           
    elseif (idBinding == MAIN_BINDING) then
        if (strCommand == "MESSAGE") then
            if (MESSAGE_HANDLER[tParams["command"]] ~= nil and type(MESSAGE_HANDLER[tParams["command"]]) == "function") then
                MESSAGE_HANDLER[tParams["command"]](tParams)
            else
                dbg("UNKNOWN | THROW AWAY: MESSAGE: " .. tParams["command"])
            end
        else
            dbg("UNKNOWN | THROW AWAY: " .. strCommand)
        end
    elseif (idBinding >= OFF_BINDING and idBinding <= HI_BINDING) then
        if (BUTTON_LINKS[strCommand]) then
            BUTTON_LINKS[strCommand](idBinding, tParams)
        end
    elseif (idBinding == TOGGLE_BINDING) then
        if (strCommand == "DO_CLICK") then
            TOGGLE_PRESET()
        end
    elseif (idBinding == CYCLE_BINDING) then
        if (strCommand == "DO_CLICK") then
            dbg("Button press on binding: CYCLE BINDING")
            if (tonumber(gFanSpeed) + 25 > 100) then
                SET_SPEED({SPEED=0})
            else
                SET_SPEED({SPEED=tonumber(gFanSpeed) + 25})
            end
        end
    else
        dbg("UNKNOWN BINDING | THROW AWAY: " .. strCommand)
    end
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--MESSAGE HANDLER (FROM MAIN REPEATER)
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

MESSAGE_HANDLER = {}

function MESSAGE_HANDLER.OUTPUT(tParams)
    if (MESSAGE_HANDLER["OUTPUT_ACTION" .. tParams["action"]] ~= nil and type(MESSAGE_HANDLER["OUTPUT_ACTION" .. tParams["action"]]) == "function") then
        MESSAGE_HANDLER["OUTPUT_ACTION" .. tParams["action"]](tParams)
    else
        dbg("UNKNOWN ACTION | THROW AWAY: " .. tParams["action"])
    end
end

function MESSAGE_HANDLER.OUTPUT_ACTION1(tParams)
    dbg("HANDLE ACTION 1: FAN_SPEED_CHANGED")
    local fanSpeed = tParams["param1"]
    gFanSpeed = tonumber(fanSpeed)
    SendToProxy(FAN_PROXY_BINDING, "CURRENT_SPEED", {SPEED = gFanSpeed / 25})
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--LIGHT PROXY COMMAND HANDLER
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

BUTTON_LINKS = {}

function BUTTON_LINKS.DO_PUSH(id, tParams)
end

function BUTTON_LINKS.DO_RELEASE(id, tParams)
end

function BUTTON_LINKS.DO_CLICK(id, tParams)
    local speed = (tonumber(id) - 200) * 25

    SET_SPEED({SPEED=speed})
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--SEND LUTRON COMMAND
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

function SendLutronCommand(strCommand, action, param1, param2)
    dbg("EXECUTE LUTRON CMD: " .. strCommand .. " ACTION: " .. action)
    local t = { command = strCommand, action = action, param1 = param1, param2 = param2 }
    C4:SendToProxy(MAIN_BINDING, "LUTRON_CMD", t)
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--FAN SPEED HANDLING
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

function GET_FAN_SPEED()
    SendLutronCommand("?OUTPUT","1")
end

function SET_SPEED(tParams)
    if (tParams ~= nil) then
        if (tParams["SPEED"] ~= nil) then
            SendLutronCommand("#OUTPUT","1", tostring(tParams["SPEED"]))
        end
    end
end

function TOGGLE_PRESET()
    if (gFanSpeed == 0) then
        SET_SPEED({SPEED = PersistData["PresetSpeed"] * 25})
    else 
        SET_SPEED({SPEED = 0})
    end
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--DEBUG
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

function startDebugTimer()
    if (g_DebugTimer) then
        g_DebugTimer = C4:KillTimer(g_DebugTimer)
    end
    g_DebugTimer = C4:AddTimer(45, "MINUTES")
end

function dbg(strDebugText)
    if (g_debugprint) then print(strDebugText) end
    if (g_debuglog) then C4:ErrorLog("Lutron_RA2_Select_Fan: " .. strDebugText) end
end

function dbgUnitTest(strDebugText)
    if (g_debugunittest) then print (strDebugText) end
    if (g_debuglog) then C4:ErrorLog("Lutron_RA2_Select_Fan: " .. strDebugText) end
end

function dbgFunction(strFunction)
    local debugFunction = true
    if (g_debugprint and debugFunction) then print("Function:\t" .. strFunction .. ":") end
    if (g_debuglog  and debugFunction) then C4:ErrorLog("Lutron_RA2_Select_Fan: " .. strFunction) end
end

--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--CLEANUP
--//////////--//////////--//////////--//////////--//////////--//////////--//////////
--//////////--//////////--//////////--//////////--//////////--//////////--//////////

-- Release things this driver had allocated...
function OnDriverDestroyed()
    -- Kill open timers
    if (g_DebugTimer ~= nil) then g_DebugTimer = C4:KillTimer(g_DebugTimer) end
end

function OnVariableChanged(strName)
    dbgFunction("OnVariableChanged(" .. strName .. ")")
end
